package com.z.toys.dns;

import com.aliyun.tea.TeaException;
import com.z.toys.dns.model.ParamModel;
import com.z.toys.dns.provider.DnsProvider;
import com.z.toys.dns.provider.DnsProviderManage;
import com.z.toys.dns.util.BeanUtil;
import com.z.toys.dns.util.StrUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.cli.*;

import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wangzz
 * @since 2024/3/26
 **/
@Slf4j(topic = "DnsMain")
public class Main {

    /**
     *
     * @param args
     * @author wangzz
     * @since 2024/3/26
     */
    @SneakyThrows
    public static void main(String[] args) {

        try {

            CommandLineParser parser = new DefaultParser();
            CommandLine cmd = parser.parse(getOptions(), args);

            String op = args[args.length - 1];
            if (StrUtil.isEmpty(op)) {
                log.error("操作参数不能为空，add(新增)/show(查询显示)/update(更新)/delete(删除)");
                return;
            }

            Map<String, String> map = Arrays.stream(cmd.getOptions()).collect(Collectors.toMap(opt -> opt.getOpt(), opt -> opt.getValue()));
            log.debug("参数：{}", map);

            ParamModel paramModel = BeanUtil.map2Bean(map, ParamModel.class);

            DnsProvider dnsProvider = DnsProviderManage.getProvider(paramModel.getProvider());
            if (Objects.isNull(dnsProvider)) {
                log.error("provider无效：{}", paramModel.getProvider());
                return;
            }

            switch (op) {
                case "show": {
                    dnsProvider.printDomainInfo(paramModel);
                    break;
                }
                case "add": {
                    dnsProvider.addDomain(paramModel);
                    break;
                }
                case "update": {
                    dnsProvider.updateDomain(paramModel);
                    break;
                }
                case "delete": {
                    dnsProvider.deleteDomain(paramModel);
                    break;
                }
                default:
                    log.error("操作参数错误：{}", op);
            }

        } catch (ParseException e) {
            log.error("参数解析错误", e);

        } catch (TeaException e) {
            log.error("TeaException", e);
        }

    }


    /**
     * 参数
     * @return
     * @author wangzz
     * @since 2024/3/26
     */
    public static Options getOptions() {

        Options options = new Options();

        Option provider = Option.builder()
                .option("provider")
                .hasArg()
                .required(true)
                .desc("dns提供商，如：aliyun")
                .build();
        options.addOption(provider);

        Option domain = Option.builder()
                .option("domain")
                .hasArg()
                .required(true)
                .desc("域名，如：engr-z.com")
                .build();
        options.addOption(domain);

        Option rr = Option.builder()
                .option("rr")
                .hasArg()
                .required(true)
                .desc("记录/子域名")
                .build();
        options.addOption(rr);

        Option type = Option.builder()
                .option("type")
                .hasArg()
                .required(false)
                .desc("记录类型，如：A/AAAA")
                .build();
        options.addOption(type);

        Option value = Option.builder()
                .option("value")
                .hasArg()
                .required(false)
                .desc("记录值")
                .build();
        options.addOption(value);

        return options;
    }

}
