package com.z.toys.dns.util;

import lombok.SneakyThrows;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.HashMap;
import java.util.Map;

/**
 * @author wangzz
 * @since 2024/3/26
 **/
public class BeanUtil {

    /**
     * 将 Bean 转换为 Map
     * @param bean
     * @return
     * @author wangzz
     * @since 2024/3/26
     */
    @SneakyThrows
    public static Map<String, Object> bean2Map(Object bean) {

        Map<String, Object> map = new HashMap<>();
        BeanInfo beanInfo = Introspector.getBeanInfo(bean.getClass());
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
        for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            map.put(propertyDescriptor.getName(), propertyDescriptor.getReadMethod().invoke(bean));
        }

        return map;
    }

    /**
     * 将 Map 转换为 Bean
     * @param map
     * @param clazz
     * @return
     * @param <T>
     * @author wangzz
     * @since 2024/3/26
     */
    @SneakyThrows
    public static <T> T map2Bean(Map<String, ?> map, Class<T> clazz) {

        // 获取 Bean 的类信息
        BeanInfo beanInfo = Introspector.getBeanInfo(clazz);

        // 获取 Bean 的属性描述符
        PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();

        // 创建 Bean 实例
        T bean = clazz.newInstance();

        // 遍历属性描述符
        for (PropertyDescriptor propertyDescriptor : propertyDescriptors) {
            // 获取属性名称
            String name = propertyDescriptor.getName();
            // 获取属性值
            Object value = map.get(name);
            // 如果属性值不为空，则将属性值设置到 Bean 中
            if (value != null) {
                propertyDescriptor.getWriteMethod().invoke(bean, value);
            }
        }

        return bean;
    }


}
